#!/usr/bin/perl -T
# $Id: search_reports.cgi,v 1.26 2018/06/15 10:45:20 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2005              ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use RFC::Main;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use CGI qw(:standard :netscape);
use Fcntl qw(:DEFAULT :flock);
use SemaforeFile;
use Sys::Hostname;

my $rig = &get_CGI_value_clean('rig');
my %access = ();
my $usermode = 0;
my $readmode = 0;
my $modifymode = 0;
if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end;
}
else
{
    #### husk at aendre accessparameter til det relevante for scriptet!! ####
    my $acmode = &get_cv('global','access_mode');
    $acmode = 'celltest' unless ($acmode);
    if (&login_ok())
    { 
	$readmode = user_auth($acmode,'r');
	$modifymode = user_auth($acmode,'m');
        if (RFC::RFCCGI::rig_mode())
        {
            ####  Read and use mode deprecated after initialisation of rig authentification #### 
            $readmode = 1;
            $usermode = 1;
            $modifymode = RFC::RFCCGI::rig_cert() unless ($modifymode);
        }
        ####
        unless (RFC::RFCCGI::rig_auth())
        {
            $readmode = 0;
            $usermode = 0;
            $modifymode = 0;
        }
	if ((param('action')) && (param('action') =~ /^report/i))
	{
	    if ($readmode)
	    {
		&print_report();
	    }
	    else
	    {
		print_header("Cell test reports");
		print RFC_start_html(-title=>"Cell test reports", -style=>{src=>'/celltest/RFCcontrol.css'});
		print center(h1({-class=>'Alert'},"You are not authorised to view reports for rig $rig!"));
		print end_html;
	    }
	    exit;
	}
        print_header("Cell test reports");
        print RFC_start_html(-title=>"Cell test reports", -style=>{src=>'/celltest/RFCcontrol.css'});
	top_nav_bar_start;
	top_no_button;
	top_no_button;
	top_no_button;
	tab_newrow;
	&js_back();
	top_js_return('#006600','white');
	top_no_button;
	top_no_button;
	top_no_button;
	top_nav_bar_button('main.cgi','','','To menu','s4');
	end_top_bar;
	$_ = param('action');
      CASE:
	{
	    ### Insert case statements ###
            /^view_inf/i and do {&print_info; last CASE;};
            /^search/i and do {&main_menu; &search; last CASE;};
	    ### Default response ####
    	    &main_menu;
	}	
	print end_html;
    }
    else 
    {
	&not_auth;
    }
}
exit;


sub print_report
{

    my $user = RFC::RFCCGI::get_user;
    my $pass = RFC::RFCCGI::get_pass;
    my $file = param('file');
    my $pdf = param('pdf');
    my $cell = '';
    my $short = '';
    if (($pdf) && ($pdf =~ /YES/))
    {
	$pdf = 1;
	print header(-type=>'application/pdf');
    }
    else
    {
	$pdf = 0;
	print header(-type=>'application/postscript');
    }
    my $test = $file;
    $test =~ /(\d+)test(\d+)/;
    my $rig = $1;
    $test = $2;
    $short = '_short' if ($file =~ /_short/ );
    my $infofile = $RFC::Header::publicdir.'rig'.$rig.'/'.$rig.'test'.$test.'/info.txt';
    my $i = SemaforeFile->new($infofile);
    my @lines = $i->readlines;
    for my $line (@lines)
    {
	if ($line =~ /(?:Cell|Sample) number\s+\:\s+([\d\w\-]+)/)
	{
	    $cell = $1;
	    last;
	}
    }
    my $testfile = '/home/rig'.$rig.'/'.$rig.'test'.$test.'/'.$rig.'test'.$test.'_'.$cell.$short.'.ps';
    $testfile = '/home/rig'.$rig.'/'.$rig.'test'.$test.'/'.$rig.'test'.$test.'_'.$cell.$short.'.pdf' if ($pdf);
    if (!-e($testfile))
    {
	### Revert to main file if short version does not exsist ###
	$testfile = '/home/rig'.$rig.'/'.$rig.'test'.$test.'/'.$rig.'test'.$test.'_'.$cell.'.ps';
	$testfile = '/home/rig'.$rig.'/'.$rig.'test'.$test.'/'.$rig.'test'.$test.'_'.$cell.'.pdf' if ($pdf);
    }
    my $f = SemaforeFile->new($testfile);
    my $res = join("\n",$f->readlines);
    print $res;
}




sub print_info()
{
    unless ($readmode)
    {
	print center(h1({-class=>'Alert'},"You are not authorised to view reports for rig $rig!"));
	return;
    }
    my $file = &get_CGI_value_clean('file');
    $file =~ /(\d+)test(\d+)/;
    my $rig = $1+0;
    my $test = $2+0;
    my $testfile = $RFC::Header::publicdir.'rig'.$rig.'/'.$file.'/info.txt';
    my $user = $_[1];
    my $pass = $_[2];
    my $infodir = $RFC::Header::homedir.'infofiles/';
    my $lock = $RFC::Header::homedir.'info.lock';
    my $f = SemaforeFile->new($testfile,$lock);
    my @dat = $f->readlines;
    my $proglog = $RFC::Header::publicdir.'rig'.$rig.'/'.$file.'/proglog';
    my $progf = SemaforeFile->new($proglog,$lock);
    my @proginf = $progf->readlines;;
    my $buffer = join("\n",@dat);
    print center(h2({-class=>'C'},"Cell information:"));
    print center(textarea(-name=>'ccell_inf',-value=>$buffer,-rows=>'20',-wrap=>'physical',-style=>'width:80%'));
    print p('');
    if (-e($proglog))
    {
	my $string = join("\n",@proginf);
	print center(h2({-class=>'C'},"Commands given:"));
	print center(textarea(-anmE=>'proginfo',-value=>$string,-rows=>'10',-wrap=>'physical',-style=>'width:80%'));
    }
    print p('');
    my $wdir = '/rig'.$rig.'/'.$rig.'test'.$test;
    my $dir = $RFC::Header::publicdir.$wdir;
    if (-d $dir)
    {
	my $st = 'http://'.$wdir;
	$st = 'https://'.$wdir if (CGI::https());	
	print start_form(-action=>$wdir);
	print_hidden();
	print center(submit(-name=>'Raw data',-class=>'s3',-style=>'width:200px;'));
	print end_form();
    }
}


sub main_menu
{
    print '<center>';
    print start_form(-class=>'form');
    &print_hidden();
    print "<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=10>\n";
    print '<tr><td colspan=4 class="heading"> Search for cell test standard reports </td></tr>';
    print "<TR><TH>\n";
    print 'Search for:';
    print "</TH><TH>\n";
    print textfield(-name=>'search',-value=>'',-autofocus=>1);
    print "</TH><TH>\n";
    print submit(-name=>'action',-value=>'search',-class=>'s3');
    print "</TH></TR></TABLE>\n";
    print end_form;
    print start_form;
    &print_hidden();
    print hidden(-name=>'search',-value=>'test',-force=>'1');
    print hidden(-name=>'action',-value=>'search',-force=>'1');
    print submit(-class=>'s2',-name=>'dummy',-value=>'Show all tested cells',-style=>'width:250px;');
    print end_form;
    print '</center>';
}    


sub search
{
    my $user = RFC::RFCCGI::get_user;
    my $pass = RFC::RFCCGI::get_pass;
    my @list = ();
    my $listserver = get_cv('servers','listserver');
    if ($listserver)
    {
        chomp $listserver;
        my @l = split(/\t+/,&RFC::Main::report_client($listserver,'servers'));
        foreach (@l)
        {
            if (/([\w\.\-]+)/)
            {
                push @list,$1;
            }
        }
    }
    @list = (get_cv('servers','server_names')) unless (@list);
    my $st = param('search');
    $st =~ /([\w\-\(\)\\\+]+)/;
    $st = $1;
    my $port = 4040;
    for my $hostname (@list)
    {
	$hostname =~ /([\w\.\-]+)/;
	$hostname = $1;
	my $string = &RFC::Main::report_client($hostname,'search',$st);
	my @res = ();
	@res = split(/\n+/,$string) if ($string);
	&print_search_tabel($user,$pass,$hostname,@res);
    }
}


sub print_search_tabel()
{
    my $user = shift(@_);
    my $pass = shift(@_);
    my $server = shift(@_);
    my $bline;
    my $linenr = 0;
    my @rows = ();
    while ($bline = shift(@_))
    {
        if (length($bline) > 10)
        {
            my $report = 0;
            my $repf = 0;
            my ($line,$rs) = split(/\s*\#\s*/,$bline);
	    my $pdf = 0;
            if (($rs =~ /REPORT/) || ($rs =~ /PDF/))
            {
                $report = 1;
                if ($rs =~ /FINISHED/)
                {
                    $repf = 1;
                }
		$pdf = 1 if ($rs =~ /PDF/);
            }
            my $string = '';
            my @dat = ();
            my @data = split(/\s+/,$line);
            $dat[0] = join(' ',$data[1],$data[2],$data[3]);
	    my $userallow = 0;
	    if ($data[5] && $data[5] =~ /(\d+)test/)
	    {
		my $r = $1;
		eval {
		    $userallow = &get_rig_auth($r,$server);
		};
	    }
            $string = &push_button($data[5],$user,$pass,$server);
	    if ($userallow && $modifymode)
	    {
		if ($pdf)
		{
		    $string = $string.&push_button_report_pdf($data[5].'_short',$user,$pass,$data[6],$report,$repf,$server);
		    $string = $string.&push_button_report_pdf($data[5],$user,$pass,$data[6],$report,$repf,$server);
		}
		else
		{
		    $string = $string.&push_button_report($data[5].'_short',$user,$pass,$data[6],$report,$repf,$server);
		    $string = $string.&push_button_report($data[5],$user,$pass,$data[6],$report,$repf,$server);
		}
	    }
	    else
	    {
		if ($report)
		{
		    $string .= '<th>Report</th><th>Report</th>';		    
		}
		else
		{
		    $string .= '<th>No report</th><th>No report</th>';		    
		}
	    }
            push @dat,$data[4];
            for (my $x = 6;$x<scalar(@data);$x++)
            {
                push @dat,$data[$x];
            }
	    foreach (@dat)
	    {
		$_ = xss($_);
	    }
	    if (@dat > 5)
	    {
		for (my $x = 0;$x < 5;$x++)
		{
		    $string = join('',$string,td[$dat[$x]]);
		}
		$string .= ' '. td[join(' ',@dat[5..scalar(@dat)])];
	    }
	    else
	    {
		for (my $x = 0;$x < scalar(@dat);$x++)
		{
		    $string = join('',$string,td[$dat[$x]]);
		}
	    }
            push(@rows,$string);
            $linenr++;
        }
    }
    if ($linenr > 0)
    {
        print '<CENTER>';
        print
            table({-border=>'0',-cellspacing=>5},
                  Tr([@rows]));
        print '</CENTER>';
    }
    print h2({-class=>'C'},"Total number of matches on $server : $linenr");
}

sub push_button($$$)
{
    my $testfile = $RFC::Header::homedir.'infofiles/'.$_[0].'.inf';
    my $user = $_[1];
    my $pass = $_[2];
    my $server = $_[3];
    my $hostname = hostname;
    $hostname =~ /([\w\-]+)/;
    $hostname = $1;
    my $string = '<TD><CENTER>';
    if ($hostname eq $server)
    {
        $string = join('',$string,start_form);
    }
    else
    {
	my $st = 'http://'.$server.'/cgi-bin/celltest/search_reports.cgi';
	$st = 'https://'.$server.'/cgi-bin/celltest/search_reports.cgi' if (CGI::https());
        $string = join('',$string,start_form(-action=>$st,-class=>'form'));
    }
    $string = join('',$string,hidden(-name=>'time',-value=>&RFC::RFCCGI::get_logout_time,-force=>'1'));
    $string = join('',$string,hidden(-name=>'jscheck',-value=>'1',-force=>'1'));
    $string = join('',$string,hidden(-name=>'action',-value=>'view_inf',-force=>'1'));
    $string = join('',$string,hidden(-name=>'file',-value=>$_[0],-force=>'1'));
    $string = join('',$string,hidden(-name=>'name',-value=>$user,-force=>'1'));
    $string = join('',$string,hidden(-name=>'pass',-value=>$pass,-force=>'1'));
    $string = join('',$string,hidden(-name=>'search'));
    $string = join('',$string,submit(-class=>'s8',-name=>'view',-value=>$_[0]));
    $string = join('',$string,end_form);
    $string = join('',$string,'</CENTER></TD>');
    return $string;
}


sub push_button_report($$$$)
{
    my $test = $_[0];
    my $user = $_[1];
    my $pass = $_[2];
    my $cell = $_[3];
    my $rep = $_[4];
    my $fin = $_[5];
    my $server = $_[6];
    my $hostname = hostname;
    $hostname =~ /([\w\-]+)/;
    $hostname = $1;
    my $string = '';
    if ($rep)
    {
        my $class = 's4';
        if ($fin)
        {
            $class = 's3';
        }
        $string = '<TD><CENTER>';
        if ($hostname eq $server)
        {
            $string = join('',$string,start_form);
        }
        else
        {
            my $st = 'http://'.$server.'/cgi-bin/celltest/search_reports.cgi';
	    $st = 'https://'.$server.'/cgi-bin/celltest/search_reports.cgi' if (CGI::https());
            $string = join('',$string,start_form(-action=>$st));
        }
	$string = join('',$string,hidden(-name=>'jscheck',-value=>'1',-force=>'1'));
	$string = join('',$string,hidden(-name=>'action',-value=>'report',-force=>'1'));
        $string = join('',$string,hidden(-name=>'time',-value=>&RFC::RFCCGI::get_logout_time,-force=>'1'));
        $string = join('',$string,hidden(-name=>'file',-value=>$_[0],-force=>'1'));
	$string = join('',$string,hidden(-name=>'name',-value=>$user,-force=>'1'));
	$string = join('',$string,hidden(-name=>'pass',-value=>$pass,-force=>'1'));
        $string = join('',$string,hidden(-name=>'search'));
        my $val = 'Report';
        $val = 'Full version' if ($_[0] !~ /_short/);
        $string = join('',$string,hidden(-name=>'view',-value=>'report',-force=>'1'));
        $string = join('',$string,submit(-class=>$class,-name=>'abcedfg',-value=>$val));
        $string = join('',$string,end_form);
        $string = join('',$string,'</CENTER></TD>');
    }
    else
    {
        $string = '<TD><CENTER>  </CENTER></TD>';
    }
    return $string;
}

sub push_button_report_pdf($$$$)
{
    my $test = $_[0];
    my $user = $_[1];
    my $pass = $_[2];
    my $cell = $_[3];
    my $rep = $_[4];
    my $fin = $_[5];
    my $server = $_[6];
    my $hostname = hostname;
    $hostname =~ /([\w\-]+)/;
    $hostname = $1;
    my $string = '';
    if ($rep)
    {
        my $class = 's4';
        if ($fin)
        {
            $class = 's3';
        }
        $string = '<TD><CENTER>';
        if ($hostname eq $server)
        {
            $string = join('',$string,start_form);
        }
        else
        {
            my $st = 'http://'.$server.'/cgi-bin/celltest/search_reports.cgi';
	    $st = 'https://'.$server.'/cgi-bin/celltest/search_reports.cgi' if (CGI::https());
            $string = join('',$string,start_form(-action=>$st));
        }
	$string = join('',$string,hidden(-name=>'jscheck',-value=>'1',-force=>'1'));
	$string = join('',$string,hidden(-name=>'action',-value=>'report',-force=>'1'));
        $string = join('',$string,hidden(-name=>'time',-value=>&RFC::RFCCGI::get_logout_time,-force=>'1'));
        $string = join('',$string,hidden(-name=>'file',-value=>$_[0],-force=>'1'));
	$string = join('',$string,hidden(-name=>'name',-value=>$user,-force=>'1'));
	$string = join('',$string,hidden(-name=>'pass',-value=>$pass,-force=>'1'));
	$string = join('',$string,hidden(-name=>'pdf',-value=>'YES',-force=>'1'));
        $string = join('',$string,hidden(-name=>'search'));
        my $val = 'Report';
        $val = 'Full version' if ($_[0] !~ /_short/);
        $string = join('',$string,hidden(-name=>'view',-value=>'report',-force=>'1'));
        $string = join('',$string,submit(-class=>$class,-name=>'abcedfg',-value=>$val));
        $string = join('',$string,end_form);
        $string = join('',$string,'</CENTER></TD>');
    }
    else
    {
        $string = '<TD><CENTER>  </CENTER></TD>';
    }
    return $string;
}



sub get_rig_auth($)
{
    my $rig = shift;
    unless (exists $access{$rig})
    {
	my $server = shift;
	$access{$rig} = &RFC::Main::report_client($server,'rig_auth',$rig,&get_user());
    }
    return $access{$rig};
}
